/*
	selectortest.js
	Copyright (C) 2005 H. Adam Mill II
	
	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License.
	You may obtain a copy of the License at
	
		 http://www.apache.org/licenses/LICENSE-2.0
	
	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/

// Get output area.
var output = document.getElementById("output");

// Run test.
var jstest = new JsTest();
jstest.test(function() {

	// These should not be found and should fail.
	var failureTests = [
			// ********* Invalid or can never happen. *********
			{
				selector : "body :root",
				size : 0
			},
			{ // This one works in IE6 CSS.
				selector : "* html",
				size : 0
			},
			{
				selector : ":root html",
				size : 0
			},
			{ // Missing leading quote.
				selector : "a[href*=fallengods\"]",
				size :  null
			},
			{ // Two commas.
				selector : ".blue,,.white",
				size : null
			},
			// ********* Valid somewhere, but not on this test page. *********
			{
				selector : "#transparent",
				size : 0
			},
			{
				selector : ".clear",
				size : 0
			},			
			{
				selector : "blink",
				size : 0
			},
			{
				selector : "a[title~=\"nine\"]",
				size :  0
			},
			{
				selector : "#tests > #chocolate",
				size : 0
			},
			{
				selector : "#chocolate #tests",
				size : 0			
			}
		];

	// These should be found and pass.
	var successTests = [
			// ********* ID *********
			{
				selector : "#floralwhite",
				size :  1
			},
			{
				selector : "#dodgerblue",
				size :  1
			},
			{
				selector : "#deepskyblue",
				size :  1
			},
			{
				selector : "#darkkhaki",
				size :  1
			},
			{
				selector : "#bright-white",
				size :  1
			},			
			// ********* Class *********
			{
				selector : ".blue",
				size :  2
			},
			{
				selector : ".links",
				size :  1
			},
			{
				selector : ".grayscale-color",
				size :  1
			},			
			// ********* Tag *********
			{
				selector : "ul",
				size :  1
			},			
			{
				selector : "li",
				size :  3
			},
			{
				selector : "p",
				size : 2
			},
			// ********* Psudo *********
			{
				selector : ":root",
				size :  1
			},
			// ********* Attribute *********
			{ // Starts with.
				selector : "a[href^=\"http://\"]",
				size :  2
			},
			{ // Equals.
				selector : "a[href=\"mailto:fallengods0@gmail.com\"]",
				size :  1
			},
			{ // Ends with.
				// NOTE: Opera automatically adds a trailing slash to href domain names 
				//		that don't have one, so don't test for just ".com".
				selector : "a[href$=\".com/\"]",
				size :  2
			},
			{ // Contains.
				selector : "a[href*=\"fallengods\"]",
				size :  2
			},
			{ // Contains w/ a closing bracket.
				selector : "a[href*=\"80]\"]",
				size :  1
			},
			{ // Contains w/ a comma.
				selector : "a[href*=\"0,\"]",
				size :  1
			},
			{ // One-of.
				selector : "a[title~=\"two\"]",
				size :  1
			},
			{ // Any.
				selector : "a[href]",
				size :  5
			},
			{ // Starts-with, hyphen-separated.
				selector : "a[title|=\"en\"]",
				size :  1
			},
			// ********* All *********
			{
				selector : "#darkkhaki *",
				size :  2
			},
			{
				selector : "body * p",
				size :  2
			},
			// ********* Subelements *********
			{
				selector : "html body div table tr td",
				size :  12
			},
			{ // Should match above test.
				selector : "html div table td",
				size :  12
			},
			{
				selector : "#darkkhaki span",
				size : 2
			},
			{ // Same as above test, only with 2 tabs instead of space.
				selector : "#darkkhaki\t\tspan",
				size : 2
			},			
			{
				selector : ".hrefs a",
				size : 4
			},
			// ********* Direct Child *********			
			{
				selector : "#darkkhaki > span",
				size :  2
			},
			{ // Should match above test.
				selector : "#darkkhaki>span",
				size :  2
			},
			{
				selector : "div > a",
				size : 4
			},
			{
				selector : "body > * > p",
				size :  1
			},
			// ********* Self *********
			{
				selector : "p#darkkhaki",
				size : 1
			},
			{
				selector : "div.anchors",
				size : 1
			},
			{ // Should match above test.
				selector : "body div.anchors",
				size : 1
			},
			// ********* Sibling *********
			{
				selector : "p + div",
				size : 1
			},
			{ // Should match above test.
				selector : "p+div",
				size : 1
			},			
			{
				selector : "th + td",
				size : 3
			},
			{
				selector : "a + br + a",
				size : 2
			},
			{
				selector : "* + div",
				size : 9
			},
			{
				selector : "p + .red",
				size : 1
			},
			{
				selector : "p + #firebrick",
				size : 1
			},
			{
				selector : "p + *",
				size : 1
			},
			// ********* Groups *********
			{
				selector : ".blue, .white",
				size : 4			
			},
			{ // Should match above test.
				selector : ".white, .blue",
				size : 4			
			},
			{ // Should also match above test.
				selector : ".white,.blue",
				size : 4
			},
			{ // These all refer to the same element, so only one is found.
				selector : "p + *, p + #firebrick, p + .red", 
				size : 1
			}
		];

	// These should be found and pass, though they aren't really valid. Just a result of the compiler being too lax.
	//		If any changes breaks these it should be ok. These are just checks for the compiler strictness.
	// 	Remove any of these tests as the compiler is improved.
	var passButInvalidTests = [
			{
				selector : "#darkkhaki > > > span",
				size : 2
			},
			{ // Should match above test.
				selector : "#darkkhaki>>span",
				size : 2
			},
			{
				selector : "th + + td",
				size : 3
			},
			{ // Should match above test.
				selector : "th>+td",
				size : 3
			}
		];

	// Can elements be compared? If not then duplicate element removal would fail.
	jstest.assert(
			document.getElementsByTagName("body")[0] == document.getElementsByTagName("body")[0],
			"body == body"
		);	

	// Success tests.
	runSelectorTest(successTests, "S: ");
	
	// Pass-but-invalid tests.
	runSelectorTest(passButInvalidTests, "I: ");

	// Pass-but-invalid tests.
	runSelectorTest(failureTests, "F: ");

	// Run tests based on a selector/size list.
	function runSelectorTest(testlist, prefix)
	{
		if(null == prefix) prefix = "";
		for(var loop=0; loop < testlist.length; loop++)
		{
			var current = testlist[loop];
			jstest.assert(function() {
						// Same as: var elementlist = DOMUtils.getElementsBySelector(document, current.selector);
						var compiled = DOMUtils.getCompiledSelector(current.selector);
						var elementlist = DOMUtils.getElementsByCompiledSelector(document, compiled);
						
						outputCompiled(compiled, current.selector);
						if(!elementlist && null != current.size)
							throw "Null element list returned.";
						else if(elementlist && elementlist.length != current.size)
							throw "Found " + elementlist.length + " element(s) was expecting " + current.size + ".";
						else
							return true;
					},
					prefix + current.selector
				);	
		}	
	}	

});
jstest.report();

// Output a compiled selector to the output area.
function outputCompiled(compiledlist, selector)
{
	if(compiledlist)
	{
		for(var outer=0; outer < compiledlist.length; outer++)
		{
			if(0 != outer)
				output.value += "#############\n" // Selector divider.
			var compiled = compiledlist[outer];
			for(var loop=0; loop < compiled.length; loop++)
			{
				if(0 != loop)
					output.value += "========\n" // Selector part divider.
				var item = compiled[loop];
				for(var prop in item)
					output.value += prop + " : " + item[prop] + "\n";
			}
		}
	}
	output.value += "\n*****************************************\n\n"; // Test divider.
}

// Show the output area and hide the show link.
function showOutput()
{
	var output = document.getElementById("output");	
	output.style["display"] = "inline";
	var show = document.getElementById("show");	
	show.style["display"] = "none";
}
